/*!
 * simpleRotation.js
 * author: Brent M. Dingle
 * last modified: 2016
 *
 * Main function of interest is likely: drawRotatedImage
 *
 * Aside: If nothing happens you likely have a "this." missing
 */
 
// ----------------------------------------------------------------------------
//                                                          GLOBAL VARs
// ----------------------------------------------------------------------------
// The below must match the id's used in the HTML file
// ----------------------------------------------------------------------------
var CANVAS_GAME_ID    = "canvasGame";

var CANVAS_WIDTH      = 640;
var CANVAS_HEIGHT     = 360;

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// theGame Object											theGame Object
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
var theGame = 
{
	// ----------------------------------------------------------------------------
	// Pseudo-constants
	// ----------------------------------------------------------------------------
	STATE_PLAYING:      5,
    
	// ----------------------------------------------------------------------------
	// Game Specific Variables
	// ----------------------------------------------------------------------------
	gameState:      this.STATE_PLAYING,   // check Init for actual value
	isInitialized:  0,   // 0=false, 1=true to indicate if Init() function was called

    startTime: 0,
    curTime:   0,
    prevTime:  0,
    delay:     0.02, //0.03,  // 0 means run as fast as possible

    gameCanvas: null,  // value is null, but type is an object
    gameCTX:    null,  // value is null, but type is an object
    
    width:  CANVAS_WIDTH,
    height: CANVAS_HEIGHT,
    
    playerX: 300,   // current player X  (blue dot)
    playerY: 150,   // current player Y
    mX: 300,        // current mouse x
    mY: 150,        // current mouse y

    playerImage: null,

    
	// ----------------------------------------------------------------------------
	// ----------------------------------------------------------------------------
	Init: function ()
	{
        // Init Local Canvas variables
		this.gameCanvas = document.getElementById(CANVAS_GAME_ID);
		this.gameCTX = this.gameCanvas.getContext("2d");
        
        this.height = this.gameCanvas.height;
        this.width  = this.gameCanvas.width;
        
        this.playerImage = new Image();
        this.playerImage.src = "spaceshipA_64x64.png";  // should be small enuf to load fast
        
        // Reset all other variables for beginning of play
        // Note difference between initialize and "play again"
        theGame.ResetPlay();
        
        // Set state to playing (may need to also make the gameplay canvas visible)
        this.gameState = this.STATE_PLAYING;
        //this.gameCanvas.style.display = 'block';
        
        this.isInitialized = 1;
    },
    
	// ----------------------------------------------------------------------------
	// ----------------------------------------------------------------------------
    ResetPlay: function()
    {
        this.playerX = 300;
        this.playerY = 150;
        this.mX = 300;
        this.mY = 150;
        this.tailpts = [];
    },
    
	// ----------------------------------------------------------------------------
    // deltaTime expected to be in seconds
	// ----------------------------------------------------------------------------
    UpdatePlay: function(deltaTime)
    {
        // If we allow window resize and auto adjust - must also
        // update this.height and width (either here or onResize)
        this.height = this.gameCanvas.height;
        this.width  = this.gameCanvas.width;

        // Clear the screen
        this.gameCTX.fillStyle = "#000";
        this.gameCTX.fillRect(0, 0, this.width, this.height);

        var e = this.mX - this.playerX; // Calculate x and y distance from player to mouse
        var f = this.mY - this.playerY;
        var angle = Math.atan2(f, e);      // angle from playerObject to mouse cursor, measured from x-axis in radians
        
        // The below 2 lines will add motion
        //this.playerX = Math.min(this.playerX + e * 0.1, this.width);   // Track towards the mouse
        //this.playerY = Math.min(this.playerY + f * 0.1, this.height);

        // ---------------------------------------------------------------
        // Draw the player using an image
        // ---------------------------------------------------------------
        theGame.drawRotatedImage(this.gameCTX, this.playerImage, 
                                 this.playerX, this.playerY, angle);

        // ---------------------------------------------------------------
        // Draw the player as a Dot  ---------------------- DEBUG
        // ---------------------------------------------------------------
        // this.gameCTX.fillStyle = '#5af';
        // this.gameCTX.beginPath();
            // this.gameCTX.arc(this.playerX, this.playerY, 4, 0, Math.PI*2, 1);
        // this.gameCTX.fill();

    },

	// ----------------------------------------------------------------------------
    // drawRotatedImage
    // context = context to draw on
    // image = an image
    // (x, y) desired position on canvas
    // angle = amount rotated, in radians
    //
    // Recall degrees to radians: radians = degrees * Math.PI/180.0; 
    //
    // Suggest reading the operations from bottom to top
    // i.e. start at command just above context.restore 
    // --> drawImage
    // --> translate to center point of image
    // --> rotate
    // --> translate to the desired position
	// ----------------------------------------------------------------------------
    drawRotatedImage: function(context, image, x, y, radAngle) 
    { 
        // Save the current coordinate system before messing with it
        context.save(); 

        // Move to the desired position to draw our image
        context.translate(x, y);

        // Rotate around the center of our image, parameter must be in radians
        context.rotate(radAngle);

        // Translate to center of image 
        //     assumes full image will be displayed, 
        //             else use 1/2 of desired width and height
        context.translate( -(image.width/2), -(image.height/2));

        // Draw the image
        // --- scaling image will work also, just adjust below translate as needed
        context.drawImage(image, 0, 0);

        // and restore the coordinate system back to what it was
        context.restore(); 
    },
    
	// ----------------------------------------------------------------------------
	// ----------------------------------------------------------------------------
    ProcessMouseMove: function(evt)
    {
        var gameCanvas = document.getElementById(CANVAS_GAME_ID);
        var mousePos = theGame.getMousePos(gameCanvas, evt);
        this.mX = mousePos.x;
        this.mY = mousePos.y;
    },
    

	// ------------------------------------------------------------------------
	// ------------------------------------------------------------------------
	getMousePos: function(canvas, evt)
	{
		var rect = canvas.getBoundingClientRect();
		return 	{
					x: Math.round( (evt.clientX-rect.left)/(rect.right-rect.left)*canvas.width ),
					y: Math.round( (evt.clientY-rect.top)/(rect.bottom-rect.top)*canvas.height )
				};
	},
	
	// ----------------------------------------------------------------------------
	// ----------------------------------------------------------------------------
    ProcessOnTouch: function(evt)
    {
        var gameCanvas = document.getElementById(CANVAS_GAME_ID);
        var touchPos = theGame.getTouchPos(gameCanvas, evt);
        this.ProcessTouchClick(touchPos.x);
    },
    
	// ------------------------------------------------------------------------
	// ------------------------------------------------------------------------
	getTouchPos: function(canvas, evt)
	{
		//var touch = evt.targetTouches[0];
		var touch = evt.changedTouches[0];
		var rect = canvas.getBoundingClientRect();
		return 	{
					x: Math.round( (touch.pageX-rect.left)/(rect.right-rect.left)*canvas.width ),
					y: Math.round( (touch.pageY-rect.top)/(rect.bottom-rect.top)*canvas.height )
				};
	},
    
	// ----------------------------------------------------------------------------
    // Update the game
    // This function is here to allow a game state to be processed
    // At the moment gameState is always STATE_PLAYING
    //
    // Also notice this.delay can be used to "force" a specific 'fastest' rate possible
    // Set this.delay to zero to run 'as fast as possible'
	// ----------------------------------------------------------------------------
	Update: function ()
	{
		if (this.gameState == this.STATE_PLAYING)
		{
            newDate  = new Date();
            this.curTime  = newDate.getTime();  // end time in milliseconds
            deltaTime = (this.curTime - this.prevTime) / 1000.0;    // delta time in seconds
            
            if (deltaTime >= this.delay)
            {
                this.prevTime = this.curTime;
                theGame.UpdatePlay(deltaTime);
            }
		}
        else
        {
            //console.log("ERROR: Unrecognized game state in Update")
        }
    },

};   // end of theGame variable/object

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// Window Handling Stuff
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
window.addEventListener("mousemove", doMouseMove, false);



// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
//
// 'global' functions
//
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
//                                                                doMouseMove
// ----------------------------------------------------------------------------
function doMouseMove(e)
{
    if (theGame.gameState == theGame.STATE_PLAYING)
    {
        theGame.ProcessMouseMove(e);
    }
};

// ----------------------------------------------------------------------------
//                                                        window.ONLOAD
//
// This function is where everything begins
// It is 'automatically' called by the browser when it believes it 
// has downloaded everything for the given current HTML page
// ----------------------------------------------------------------------------
window.onload = function()
{
	// Initialize the game
	theGame.Init();

	// Run the game (start the game loop)
	runGame();
};

// ----------------------------------------------------------------------------
//                                                         the game 'loop'
// Event handlers/listeners cover the game input
// ----------------------------------------------------------------------------
function runGame()
{
	theGame.Update();

    // Be certain to call runGame again	
	requestAnimationFrame(runGame);
};

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// end of file
    
